@@ -1,11 +1,11 @@
 """   Tests for the module "dcm2bidsphysio.py"   """
 
-from pathlib import Path
 import sys
 
 import pytest
 
 from bidsphysio.dcm2bids import dcm2bidsphysio as d2bp
+from bidsphysio.base.utils import check_bidsphysio_outputs
 from .utils import TESTS_DATA_PATH
 
 '''
@@ -18,6 +18,7 @@ TO-DO:
   changing the tests in this file
 '''
 
+
 ###   Fixtures   ###
 
 @pytest.fixture
@@ -28,6 +29,7 @@ def mock_dcm2bidsphysio(monkeypatch):
        actually running anything: just the instructions in the runner
        before the call to dcm2bids
     """
+
     def mock_dcm2bids(*args, **kwargs):
         print('mock_dcm2bids called')
         return
@@ -35,7 +37,6 @@ def mock_dcm2bidsphysio(monkeypatch):
     monkeypatch.setattr(d2bp, "dcm2bids", mock_dcm2bids)
 
 
-
 ###   Tests   ###
 
 def test_main_args(
@@ -52,12 +53,12 @@ def test_main_args(
     # 1) "infile" doesn't exist:
     infile = str(tmpdir / 'boo.dcm')
     args = (
-        'dcm2bidsphysio -i {infile} -b {bp}'.format(
+        'dcm2bidsphysio -b {bp} -i {infile}'.format(
             infile=infile,
             bp=tmpdir / 'mydir' / 'foo'
         )
     ).split(' ')
-    monkeypatch.setattr(sys, 'argv',args)
+    monkeypatch.setattr(sys, 'argv', args)
     with pytest.raises(FileNotFoundError) as e_info:
         d2bp.main()
     assert str(e_info.value).endswith(' file not found')
@@ -65,12 +66,20 @@ def test_main_args(
 
     # 2) "infile" does exist, but output directory doesn't exist:
     #    The output directory should be created and the "dcm2bids" function should be called
-    args[ args.index('-i')+1 ] = str(TESTS_DATA_PATH / 'samplePhysio+02+physio_test+00001.dcm')
-    monkeypatch.setattr(sys, 'argv',args)
+    args[args.index('-i') + 1] = str(TESTS_DATA_PATH / 'samplePhysio+02+physio_test+00001.dcm')
+    monkeypatch.setattr(sys, 'argv', args)
     d2bp.main()
     assert (tmpdir / 'mydir').exists()
     assert capfd.readouterr().out == 'mock_dcm2bids called\n'
 
+    # 3) "infile" contains more than one file:
+    args.append(
+        str(TESTS_DATA_PATH / 'samplePhysio+01+physio_test_01+00002.dcm')
+    )
+    monkeypatch.setattr(sys, 'argv', args)
+    # Make sure 'main' runs without errors:
+    assert d2bp.main() is None
+
 
 def test_parse_log():
     """   Test for parse_log   """
@@ -92,15 +101,14 @@ def test_parse_log():
     simulated_string += '{t} PULS {s}\n'.format(t=expected_times[0], s=expected_signals[0])
     simulated_string += '{t} RESP {s}\n'.format(t=expected_times[1], s=expected_signals[1])
 
-    # generate a binary string, and parse it:
-    log_bytes = bytearray()
-    log_bytes.extend(simulated_string.encode())
-    waveform_name, t, s, dt = d2bp.parse_log(log_bytes)
+    # generate a string, line-by-line, and parse it:
+    physio_log_lines = simulated_string.splitlines()
+    waveform_name, t, s, dt = d2bp.parse_log(physio_log_lines)
 
     assert waveform_name == expected_LogDataType
-    assert all(t == expected_times)
+    assert all(t == [2.5 * e for e in expected_times])    # the "SampleTime" is in units of 2.5 ms
     assert all(s == expected_signals)
-    assert dt == expected_SampleTime
+    assert dt == 2.5 * expected_SampleTime   # the "SampleTime" is in units of 2.5 ms
 
     ###   Test trigger signal   ###
     expected_LogDataType = 'ACQUISITION_INFO'
@@ -118,15 +126,14 @@ def test_parse_log():
     # a new volume:
     simulated_string += '1     0       {t}      foo    0\n'.format(t=expected_times[2])
 
-    # generate a binary string, and parse it:
-    log_bytes = bytearray()
-    log_bytes.extend(simulated_string.encode())
-    waveform_name, t, s, dt = d2bp.parse_log(log_bytes)
+    # generate a string, line-by-line, and parse it:
+    physio_log_lines = simulated_string.splitlines()
+    waveform_name, t, s, dt = d2bp.parse_log(physio_log_lines)
 
     assert waveform_name == expected_LogDataType
-    assert all(t == expected_times)
+    assert all(t == [2.5 * e for e in expected_times])    # the "SampleTime" is in units of 2.5 ms
     assert all(s == expected_signals)
-    assert dt == expected_SampleTime
+    assert dt == 2.5 * expected_SampleTime   # the "SampleTime" is in units of 2.5 ms
 
     # Note: if parse_log tried to incorrectly add the "don't log this: repeated echo"
     #       as time, it would give a "ValueError: invalid literal for int() with base 10"
@@ -142,19 +149,18 @@ def test_dcm2bids(
     We will call it by calling "main" to make sure the output directory
     is created, etc.
     """
-    import json
-    import gzip
-
-    infile = str(TESTS_DATA_PATH / 'samplePhysio+02+physio_test+00001.dcm')
     outbids = str(tmpdir / 'mydir' / 'bids')
 
+    # 1) Single DICOM infile:
+    print('Testing a single DICOM file...')
+    infile = str(TESTS_DATA_PATH / 'samplePhysio+02+physio_test+00001.dcm')
     args = (
         'dcm2bidsphysio -i {infile} -b {bp} -v'.format(
             infile=str(infile),
             bp=outbids
         )
     ).split(' ')
-    monkeypatch.setattr(sys, 'argv',args)
+    monkeypatch.setattr(sys, 'argv', args)
 
     # call "main" (which will create the output dir and call "dcm2bids"):
     d2bp.main()
@@ -163,29 +169,76 @@ def test_dcm2bids(
     assert capfd.readouterr().out != 'mock_dcm2bids called\n'
 
     # Check that we have as many signals as expected (2, for this file):
-    json_files = sorted(Path(tmpdir / 'mydir').glob('*.json'))
-    data_files = sorted(Path(tmpdir / 'mydir').glob('*.tsv*'))
-    assert len(json_files)==len(data_files)==2
+    check_bidsphysio_outputs(outbids,
+                             ['cardiac', 'respiratory'],
+                             [200, 50],
+                             -4.082,
+                             TESTS_DATA_PATH / 'dcm_')
 
-    for s in ['respiratory','cardiac']:
-        expectedFileBaseName = Path(outbids).name + '_recording-' + s + '_physio'
-        expectedFileName = tmpdir / 'mydir' / expectedFileBaseName
-        assert (expectedFileName + '.json') in json_files
-        assert (expectedFileName + '.tsv.gz') in data_files
+    # 2) Two DICOM infiles: It should give an error:
+    print('Testing two DICOM files...')
+    infiles = [str(TESTS_DATA_PATH / 'samplePhysio+02+physio_test+00001.dcm'),
+               str(TESTS_DATA_PATH / 'samplePhysio+01+physio_test_01+00002.dcm')]
+    args = (
+        'dcm2bidsphysio -i {infile} -b {bp} -v'.format(
+            infile=" ".join(infiles),
+            bp=outbids
+        )
+    ).split(' ')
+    monkeypatch.setattr(sys, 'argv', args)
 
-        # check content of the json file:
-        with open(expectedFileName + '.json') as f:
-            d = json.load(f)
-            assert d['Columns'] == [ s, 'trigger']
-            assert d['StartTime'] == -1.632
-            if s == 'respiratory':
-                assert d['SamplingFrequency'] == 125
-            elif s == 'cardiac':
-                assert d['SamplingFrequency'] == 500
+    with pytest.raises(RuntimeError) as e_info:
+        d2bp.main()
+    assert 'dcm2bids can only take one DICOM file' in str(e_info.value)
 
-        # check content of the tsv file:
-        with open( TESTS_DATA_PATH / ('dcm_' + s + '.tsv'),'rt' ) as expected, \
-            gzip.open(expectedFileName + '.tsv.gz','rt') as f:
-                for expected_line, written_line in zip (expected, f):
-                    assert expected_line == written_line
+    # 3) Multiple .log infiles:
+    print('Testing multiple ".log" files...')
+    infiles = [str(TESTS_DATA_PATH / 'Physio_Info.log'),
+               str(TESTS_DATA_PATH / 'Physio_RESP.log'),
+               str(TESTS_DATA_PATH / 'Physio_PULS.log')]
+    args = (
+        'dcm2bidsphysio -i {infile} -b {bp}'.format(
+            infile=" ".join(infiles),
+            bp=outbids
+        )
+    ).split(' ')
+    monkeypatch.setattr(sys, 'argv', args)
+    d2bp.main()
 
+    # Check that we have as many signals as expected (2 in this case)
+    # (we don't check the content of the .tsv file):
+    check_bidsphysio_outputs(outbids,
+                             ['cardiac', 'respiratory'],
+                             [200, 50],
+                             -7.077,
+                             None)
+
+
+def test_timing(
+        monkeypatch,
+        tmpdir,
+        capfd
+):
+    """
+    This test checks that the timing is OK.
+    It will read the physio signals from a DICOM file and save them as BIDS.
+    Then, from the trigger signal in the physio bids files, we'll estimate
+    the TR, and compare it with the actual TR.
+    """
+    outbids = str(tmpdir / 'mydir' / 'bids')
+
+    infile = str(TESTS_DATA_PATH / 'samplePhysio+02+physio_test+00001.dcm')
+    args = (
+        'dcm2bidsphysio -i {infile} -b {bp} -v'.format(
+            infile=str(infile),
+            bp=outbids
+        )
+    ).split(' ')
+    monkeypatch.setattr(sys, 'argv', args)
+
+    # call "main" (which will create the output dir and call "dcm2bids"):
+    d2bp.main()
+
+    # now, read the TR (it is also recorded in the physio DICOM file:
+    d = d2bp.pydicom.dcmread(infile, stop_before_pixels=True)
+    assert False
\ No newline at end of file
